﻿using Hotline.SeedData;
using Hotline.Settings;
using Hotline.Share.Enums.Settings;
using Hotline.Users;
using Microsoft.Extensions.DependencyInjection;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Entities;
using XF.Domain.Exceptions;
using XF.Domain.Extensions;

namespace Hotline.Repository.SqlSugar.DataPermissions;

public class DataPermissionManager : IDataPermissionManager, IScopeDependency
{
    private readonly IServiceScopeFactory _serviceScopeFactory;
    private readonly ISessionContext _sessionContext;
    private readonly ITypedCache<IReadOnlyList<TableAccessLevel>> _cacheAccessLevel;
    private const string AccessLevelCache = "AccessLevel_";

    public DataPermissionManager(
        IServiceScopeFactory serviceScopeFactory, 
        ISessionContext sessionContext,
        ITypedCache<IReadOnlyList<TableAccessLevel>> cacheAccessLevel)
    {
        _serviceScopeFactory = serviceScopeFactory;
        _sessionContext = sessionContext;
        _cacheAccessLevel = cacheAccessLevel;
    }

    public ETableAccessLevel GetQueryFilter<TEntity>(ISessionContext sessionContext) where TEntity : class, IEntity<string>, IDataPermission, new()
    {
        //return GetHighestTableAccessLevel(sessionContext.Roles, typeof(TEntity).Name.ToSnakeCase());

        ////old
        ETableAccessLevel queryFilter = GetCurrentQueryFilter(sessionContext.Roles, typeof(TEntity).Name);
        //return new DataPermissionScheme(queryFilter);
        return queryFilter;
    }

    public (string orgId, string orgCode, string creatorId, string? areaId) GetDataPermissionOptions()
    {
        //using var scope = _serviceScopeFactory.CreateScope();
        //var userRepository = scope.ServiceProvider.GetRequiredService<IUserRepository>();
        //var user = userRepository.GetAsync(_sessionContext.RequiredUserId).GetAwaiter().GetResult();
        //if (!string.IsNullOrEmpty(user!.OrgId))
        //{
        //    var orgRepository = scope.ServiceProvider.GetRequiredService<ISystemOrganizeRepository>();
        //    var org = orgRepository!.GetAsync(user!.OrgId).GetAwaiter().GetResult();

        //    return (org!.Id, org.OrgCode, _sessionContext.RequiredUserId, org.OrgCode);
        //}
        //return ("", "", _sessionContext.RequiredUserId, "");
        return (_sessionContext.RequiredOrgId, _sessionContext.RequiredOrgCode,
            _sessionContext.RequiredUserId, _sessionContext.RequiredOrgId);
    }

    public static string GetCacheKey(string role)
    {
        return $"{AccessLevelCache}{role}";
    }

    private ETableAccessLevel GetCurrentQueryFilter(string[] roles, string entityName)
    {
        using var scope = _serviceScopeFactory.CreateScope();
        var systemDataTableRepository = scope.ServiceProvider.GetRequiredService<ISystemDataTableRepository>();
        var systemDataAuthorityRepository = scope.ServiceProvider.GetRequiredService<ISystemDataAuthorityRepository>();

        ////查询对应表配置
        var tableModel = systemDataTableRepository.GetAsync(x => x.EntityName == entityName).GetAwaiter().GetResult();
        if (tableModel == null)
            throw UserFriendlyException.SameMessage($"未配置表{entityName}的查询权限");
        ////查询表对应最高数据权限
        var auth = systemDataAuthorityRepository.GetMyTopDataAuth(roles, tableModel.Id);
        return auth;
    }

    /// <summary>
    /// 查询最高访问权限
    /// </summary>
    /// <param name="roles"></param>
    /// <param name="tableName"></param>
    /// <returns></returns>
    private ETableAccessLevel GetHighestTableAccessLevel(string[] roles, string tableName)
    {
        if (roles.Contains(RoleSeedData.AdminRole))
            return ETableAccessLevel.All;

        //get top level
        return roles.Select(d => GetTableAccessLevel(d, tableName)).Max();
    }

    private ETableAccessLevel GetTableAccessLevel(string role, string tableName)
    {
        //该方法实现：只做缓存查询，如未查到视为拒绝访问（缓存数据由系统启动时加载）
        var levels = _cacheAccessLevel.Get(GetCacheKey(role));
        if (levels == null || !levels.Any())
            return ETableAccessLevel.Deny;
        var level = levels.FirstOrDefault(d=>d.TableName == tableName);
        return level?.AccessLevel ?? ETableAccessLevel.Deny;
    }
}